#!/bin/bash

################################################################
# This script was written to create a CentOS 5.1 RPM for the 
# ISC DHCP server with the ldap patch written by Brian Masney.
################################################################
# Written by Niels Basjes - http://niels.basjes.nl
# This script may be used under GPL version 2
################################################################

DHCP_VERSION=3.0.5
RPM_RELEASE=7.el5

DHCP_SRC_RPM=dhcp-${DHCP_VERSION}-${RPM_RELEASE}.src.rpm
DHCP_SRC_RPM_URL=ftp://ftp.nluug.nl/mirror/os/Linux/distr/CentOS/5.1/os/SRPMS/${DHCP_SRC_RPM}

DHCP_LDAP_PATCH=dhcp-${DHCP_VERSION}-ldap-patch
BRIAN_MASNEY_HOMEPAGE=http://home.ntelos.net/~masneyb/
DHCP_LDAP_PATCH_URL=${BRIAN_MASNEY_HOMEPAGE}${DHCP_LDAP_PATCH}

DHCP_LDAP_WARNINGS_PATCH=dhcp-${DHCP_VERSION}-ldap.warnings-patch

BUILDDIR=`pwd`/RebuildDir

# Step 0: Check if everything we need is there
if [ -f ${DHCP_SRC_RPM} -a -f ${DHCP_LDAP_PATCH} -a -f /usr/include/ldap.h ];
then
    echo "We have all the files"
else
    echo "======================================================="
    if [ ! -f ${DHCP_SRC_RPM} ];
    then
        echo ""
        echo "You need to put the ${DHCP_SRC_RPM} in the current directory." 
        echo "You can try doing something like this: curl -O ${DHCP_SRC_RPM_URL}"
        echo ""
    fi

    if [ ! -f ${DHCP_LDAP_PATCH} ];
    then
        echo ""
        echo "You need to put the ${DHCP_LDAP_PATCH} in the current directory." 
        echo "You can try doing something like this: curl -O ${DHCP_LDAP_PATCH_URL}"
        echo ""
    fi

    if [ ! -f /usr/include/ldap.h ];
    then
        echo ""
        echo "You need install the openldap-devel package before you can build the RPM"
        echo "Do this: yum install openldap-devel"
        echo ""
    fi
    echo "======================================================="

    exit -1;
fi

# Step 1: Install the src rpm into our private directory
mkdir -p ${BUILDDIR}/BUILD  ${BUILDDIR}/RPMS  ${BUILDDIR}/SOURCES  ${BUILDDIR}/SPECS  ${BUILDDIR}/SRPMS
rpm --define="_topdir ${BUILDDIR}" -i ${DHCP_SRC_RPM}
chown -R $USER ${BUILDDIR}

# Step 2: Copy the patches to the  SOURCES directory
cp ${DHCP_LDAP_PATCH} ${BUILDDIR}/SOURCES
cat ${DHCP_LDAP_WARNINGS_PATCH}* > ${BUILDDIR}/SOURCES/${DHCP_LDAP_WARNINGS_PATCH}

# Step 3: Update the spec file to change the name of the RPM and to include the LDAP patch
[ -f ${BUILDDIR}/SPECS/dhcp.spec.original ] || mv ${BUILDDIR}/SPECS/dhcp.spec ${BUILDDIR}/SPECS/dhcp.spec.original
sed "
    # Rename the base package to avoid 'accidental' upgrades (i.e. yum update)
    s/^\(Name: .*\)/\1_ldap/g;

    # Ensure the right subdirectory is assumed for the sources (we changed the name of the RPM)
    s/\(^%setup.*$\)/\1 -n dhcp-${DHCP_VERSION}/g;

    # Add the line in site.conf which is overwritten by the default RedHat spec file
    s/^\(^INCDIR=.*$\)/\1\nLIBS=-lssl -lcrypto -lldap -llber/g;

    # Now add an additional flag to allow the usage of the deprecated LDAP functions like ldap_get_values
    s/^\(RPM_OPT_FLAGS.*-Werror.*$\)/\1\nRPM_OPT_FLAGS=\"\$RPM_OPT_FLAGS -DLDAP_DEPRECATED\"/g;

    # Remove files that are empty and related to LDAP
    # and Set the permissions right
    s/^\(%build.*$\)/\1\nfind . -type f -name '*.ldap' -size 0 | xargs -r rm\nchmod -R a+rX */g;

    # We must 'fix' the names of the patches for be 'fixed' on the actual file names    
    s/^\(Patch.*\)%{name}-/\1dhcp-/g;

    # Now insert the ldap patch and the patch to make it all work in the spec file
    s/^\(Patch15:.*$\)/\1\nPatch389: ${DHCP_LDAP_PATCH}\nPatch390: ${DHCP_LDAP_WARNINGS_PATCH}/g;
    s/^\(\%patch15.*$\)/\1\n\n# The LDAP patch by Brian Masney\n\%patch389 -p1 -b .ldap\n# The compile warning fixes for the LDAP patch by Niels Basjes\n%patch390 -p1 -b .ldapwarnings/g;

    # Now insert the ldap patch and the patch to make it all work in the spec file

    # Update the changelog
    s@^\(\%changelog.*$\)@\1\n\* $(date "+%a %b %d %Y") Niels Basjes (http://niels.basjes.nl)\n- Intergrated Brian Masney\'s ldap patch ${BRIAN_MASNEY_HOMEPAGE}\n@g;

" < ${BUILDDIR}/SPECS/dhcp.spec.original > ${BUILDDIR}/SPECS/dhcp.spec

#Step 4: Rebuild the RPM
rpmbuild --define="_topdir ${BUILDDIR}" -ba ${BUILDDIR}/SPECS/dhcp.spec

#Step 5: We're done
#clear
echo "============================================="
cd ${BUILDDIR}/RPMS/*
echo "Here are your new RPMS"
pwd
echo "============================================="
ls -laF
echo "============================================="
